/********************************************************************************
*                                                                               *
*                      F X   M O V I E   M A N A G E R                          *
*                                                                               *
*********************************************************************************
* Copyright(C) 2008,2009 by Luigi Ziviani <ziviani_luigi@yahoo.it>.             *
* All Rights Reserved.                                                          *
*********************************************************************************
* Id: mimetypedialog.cpp,   v 1.5                      2008/01/01               *
*********************************************************************************
* This program is free software; you can redistribute it and/or modify          *
* it under the terms of the GNU General Public License as published by          *
* the Free Software Foundation; either version 3 of the License, or             *
* (at your option) any later version.                                           *
*                                                                               *
* This program is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
* GNU General Public License for more details.                                  *
* You should have received a copy of the GNU General Public License             *
* along with this program; if not, write to the Free Software                   *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA     * 
********************************************************************************/

#include <xincs.h>
#include <fx.h>
#include "MimeTypeDialog.h"
#include "fxmoviemanager.h"
#include "icons.h"


FXDEFMAP(MimeTypeDialog) MimeTypeDialogMap[]={
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_ICONPATH,		MimeTypeDialog::onCmdSelectIconPath),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_COMMAND,		MimeTypeDialog::onCmdSelectCommand),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_SELECT_MIMETYPE,		MimeTypeDialog::onCmdSelectMimeType),
  FXMAPFUNC(SEL_CLOSE,	MimeTypeDialog::ID_FXMMAPPLICATION,		MimeTypeDialog::onCmdCloseDialog),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_CREATE_MIMEASSOCIATION,	MimeTypeDialog::onCmdCreateFileAssociation),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_DELETE_MIMEASSOCIATION,	MimeTypeDialog::onCmdDeleteFileAssociation),
  FXMAPFUNC(SEL_COMMAND,MimeTypeDialog::ID_RENAME_MIMEASSOCIATION,	MimeTypeDialog::onCmdRenameFileAssociation),
  FXMAPFUNC(SEL_CHANGED,MimeTypeDialog::ID_SELECT_MIMEASSOCIATION,	MimeTypeDialog::onCmdFileAssociation),
  FXMAPFUNCS(SEL_COMMAND,MimeTypeDialog::ID_SELECT_ICONBIG_NAME,MimeTypeDialog::ID_SELECT_ICONSMALL_NAME,MimeTypeDialog::onCmdSelectIcon),
  };


// Object implementation
FXIMPLEMENT(MimeTypeDialog,FXDialogBox,MimeTypeDialogMap,ARRAYNUMBER(MimeTypeDialogMap))


// Construct window
MimeTypeDialog::MimeTypeDialog(FXWindow *own):FXDialogBox(own,"FXMovieManager Setup",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0){


  // get iconpath, theme and language Settings
  iconpath = getApp()->reg().readStringEntry("SETTINGS","iconpath");
  theme = getApp()->reg().readStringEntry("SETTINGS","theme");
  language = getApp()->reg().readStringEntry("SETTINGS","language");


  // Set the Main Application Target
  setTarget(this);

  // Connect Datatargets
  target_association_type.connect(fileassociation.type);
  target_association_command.connect(fileassociation.command);
  target_association_mimetype.connect(fileassociation.mimetype);
  target_iconpath.connect(iconpath);
  target_theme.connect(theme);
  target_language.connect(language);

  // Set the message identifier for this data target
  setSelector(ID_FXMMAPPLICATION);


  // Retrive theme settings
  theme=getApp()->reg().readStringEntry("SETTINGS","theme");
  theme_fore=getApp()->reg().readColorEntry("SETTINGS","forecolor");
  theme_base=getApp()->reg().readColorEntry("SETTINGS","basecolor");
  theme_back=getApp()->reg().readColorEntry("SETTINGS","backcolor");
  theme_menufore=getApp()->reg().readColorEntry("SETTINGS","selmenutextcolor");
  theme_menuback=getApp()->reg().readColorEntry("SETTINGS","selmenubackcolor");
  theme_selfore=getApp()->reg().readColorEntry("SETTINGS","selforecolor");
  theme_selback=getApp()->reg().readColorEntry("SETTINGS","selbackcolor");


  // Build icons
  logoicon=new FXPNGIcon(getApp(),fxmmlogo);
  abouticon=new FXGIFIcon(getApp(),b_list);
  conficon=new FXGIFIcon(getApp(),b_conf);
  diricon=new FXGIFIcon(getApp(),m_dir);

  darkthemeicon=new FXJPGIcon(getApp(),darktheme);
  shinethemeicon=new FXJPGIcon(getApp(),shinetheme);
  classicthemeicon=new FXJPGIcon(getApp(),classictheme);
  gentlethemeicon=new FXJPGIcon(getApp(),gentletheme);
  fogthemeicon=new FXJPGIcon(getApp(),fogtheme);
  moonthemeicon=new FXJPGIcon(getApp(),moontheme);

  // get the size of normal font and create a new font with size x 1.5
  getApp()->getNormalFont()->create();
  FXFontDesc fontdescription=getApp()->getNormalFont()->getFontDesc();
  fontdescription.size = (FXuint) (((double)fontdescription.size) * 1.5);
  titlefont = new FXFont(getApp(),fontdescription);
  titlefont->create();

  tooltip=new FXToolTip(getApp());

  // Declare reused frames
  FXHorizontalFrame *hframe=NULL;
  FXVerticalFrame   *sframe=NULL;
  FXVerticalFrame   *vframe=NULL;
  FXLabel           *label=NULL;
  FXPacker 	    	*packer;

  // main frame
  FXVerticalFrame   *mainframe=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  // Get icon margin
  const FXint spacing=(logoicon ? 10 : 20);

  // Create the header
  label=new FXLabel(mainframe," FXMovieManager Configuration",logoicon,LAYOUT_FILL_X|JUSTIFY_LEFT|TEXT_AFTER_ICON,0,0,0,0,spacing,spacing,spacing,spacing);
  label->setBackColor(FXRGB(0,0,0));
  label->setTextColor(FXRGB(255,255,0));
  label->setFont(titlefont);

  new FXSeparator(mainframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  
  FXHorizontalFrame *hmainframe=new FXHorizontalFrame(mainframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  FXVerticalFrame *buttonframe=new FXVerticalFrame(hmainframe,LAYOUT_FILL_Y|LAYOUT_LEFT|PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT,0,0,0,0, 5,5,5,5);
  new FXSeparator(hmainframe,SEPARATOR_GROOVE|LAYOUT_FILL_Y);


  FXSwitcher *switcher = new FXSwitcher(hmainframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0);

  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"About",abouticon,switcher,FXSwitcher::ID_OPEN_FIRST,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"How To",abouticon,switcher,FXSwitcher::ID_OPEN_SECOND,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"MimeTypes",conficon,switcher,FXSwitcher::ID_OPEN_THIRD,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);
  vframe = new FXVerticalFrame(buttonframe,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
  new FXButton(vframe,"Preferences",conficon,switcher,FXSwitcher::ID_OPEN_FOURTH,FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL);


  // About this program
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(hframe," About FXMovieManager",NULL,LAYOUT_CENTER_Y);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  sframe = new FXVerticalFrame(hframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,5,5,5,5,0,0);

  label = new FXLabel(vframe,
	"FXMovieManager is a FileManager with movie thumbnails,\n"
	"it manages a PlayList and act as simple front-end to MPlayer.\n"
	"Copyright (C) 2008 Luigi Ziviani (ziviani_luigi@yahoo.it).\n\n"
	"This program is free software; you can redistribute it\n"
	"and/or modify it under the terms of the GNU General Public\n"
	"License as published by the Free Software Foundation;\n"
	"either version 3 of the License, or(at your option)\n"
	"any later version.\n\n"
	"This program is distributed in the hope that it will be useful,\n"
	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
	"See the GNU General Public License for more details.\n\n"
	"You should have received a copy of the GNU General Public License\n"
	"along with this program; if not, write to the \n"
	"Free Software Foundation, Inc.,\n"
	"59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"
	);



  // Help instructions
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(hframe," How to use FXMovieManager",NULL,LAYOUT_CENTER_Y);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  sframe = new FXVerticalFrame(hframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,5,5,5,5,0,0);

  label = new FXLabel(vframe,
	"To manage thumbnails of movie files a list of buttons on the\n"
	"right of the main application area can be used.\n\n"
	"I)   To Create and Show thumbnails of movie files:\n"
	"select the movies and press the button [Create thumbnails].\n"
	"II)  To ReCreate the thumbnails:\n"
	"select the movies and press the button [Recreate thumbnails].\n"
	"III) To load a movie into the playlist:\n"
	"double click on a movie in the filelist or open the file dialog\n"
	"and select one or more movies keeping [Ctrl] or [Shift] pressed.\n"
	"IV)  To delete hidden directories that store the thumbnails:\n"
	"press the button [Delete Thumbnails]\n"
	"V)   To associate file extensions to icons, mime-types and commands:\n"
	"Select the button [MimeTypes] in the Current dialog and\n"
	"a window will be displayed where to add new file associations\n"
	"VI)  To change Theme and/or Language:\n"
	"Select the button  [Theme selection] in the Current dialog and\n"
	"a window will be displayed where to set your preferences\n\n"
	"VII) See also the manpage for keyboard enabled buttons\n" 
	);



  // Content-Types configuration
  vframe = new FXVerticalFrame(switcher,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(hframe,"Search Path for Icons:",NULL,LAYOUT_CENTER_Y);
  new FXTextField(hframe,2,&target_iconpath,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);
  new FXButton(hframe,"..",diricon,this,ID_SELECT_ICONPATH,LAYOUT_SIDE_RIGHT|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  vframe = new FXVerticalFrame(hframe,LAYOUT_FILL_Y,0,0,0,0,5,5,5,5);

  new FXSeparator(hframe,SEPARATOR_GROOVE|LAYOUT_FILL_Y);

  new FXLabel(vframe,"Extensions of File: ");
  sframe = new FXVerticalFrame(vframe,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_Y|LAYOUT_FILL_X,0,0,0,0,0,0,0,0);
  associationlist = new FXList(sframe,this,ID_SELECT_MIMEASSOCIATION,LAYOUT_FILL_Y|LAYOUT_FILL_X|LIST_BROWSESELECT);
  associationlist->setSortFunc(FXList::ascending);
  FXHorizontalFrame * listbuttonframe = new FXHorizontalFrame(vframe,PACK_UNIFORM_WIDTH|LAYOUT_FILL_X,0,0,0,0,0,0,0,0);
  new FXButton(listbuttonframe,"&Insert\tInsert a New Extension",NULL,this,ID_CREATE_MIMEASSOCIATION);
  new FXButton(listbuttonframe,"&Delete\tDelete an Extension",NULL,this,ID_DELETE_MIMEASSOCIATION);
  new FXButton(listbuttonframe,"&Change\tChange the name of an Extension",NULL,this,ID_RENAME_MIMEASSOCIATION);

  vframe = new FXVerticalFrame(hframe,LAYOUT_FILL_Y|LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);


  packer = new FXPacker(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(packer,"Type Name:");
  new FXTextField(packer,2,&target_association_type,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);


  packer = new FXPacker(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(packer,"Content-Type [object/format]:");
  mimetypelist = new FXComboBox(packer,1,this,ID_SELECT_MIMETYPE,LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|COMBOBOX_NO_REPLACE);
  mimetypelist->setSortFunc(FXList::ascending);
  mimetypelist->setNumVisible(9);
  mimetypelist->appendItem(" ");

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);


  packer = new FXPacker(vframe,LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  new FXLabel(packer,"Open With Command:");
  new FXButton(packer,"..",diricon,this,ID_SELECT_COMMAND,LAYOUT_SIDE_RIGHT|LAYOUT_CENTER_Y|ICON_AFTER_TEXT|FRAME_RAISED|FRAME_THICK);
  new FXTextField(packer,2,&target_association_command,FXDataTarget::ID_VALUE,LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_CENTER_Y|FRAME_SUNKEN|FRAME_THICK);

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);


  FXMatrix *iconsmatrix=new FXMatrix(vframe,4,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);
  new FXLabel(iconsmatrix,"Normal",NULL,LAYOUT_CENTER_X|LAYOUT_FILL_COLUMN);
  button_bigname = new FXButton(iconsmatrix,"\tChange MIME-Type Icon",NULL,this,ID_SELECT_ICONBIG_NAME,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,64,64, 1,1,1,1);
  new FXLabel(iconsmatrix,"Small",NULL,LAYOUT_CENTER_X|LAYOUT_FILL_COLUMN);
  button_smallname = new FXButton(iconsmatrix,"\tChange MIME-Type Icon",NULL,this,ID_SELECT_ICONSMALL_NAME,FRAME_RAISED|LAYOUT_CENTER_X|LAYOUT_CENTER_Y|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,64,64, 1,1,1,1);


  // import file associations **********************************************//
  FXString datastring;
  FXString mimetype;

  FXStringDict *filetypes; // The dictionary class maintains a fast-access 
                           // hash table of entities indexed by a character string.

  filetypes = getApp()->reg().find("FILETYPES"); // FILETYPES is the registry section where are contents-type data

  if(filetypes){
    for(FXint i=filetypes->first(); i<filetypes->size(); i=filetypes->next(i)){
      associationlist->appendItem(filetypes->key(i),NULL,NULL,TRUE); 	// Key string of struct FXDictEntry
      datastring = filetypes->data(i); 					// Data of struct FXDictEntry
      mimetype = datastring.section(";",4);
      if (!mimetype.empty() && (mimetypelist->findItem(mimetype)==-1)){
        mimetypelist->appendItem(mimetype);
        }
      }
    }

  associationlist->sortItems();
  mimetypelist->sortItems();
  //***********************************************************************//




  // Preferences
  hframe = new FXHorizontalFrame(switcher,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);

  vframe = new FXVerticalFrame(hframe,LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(vframe," Theme selection",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);

  combobox1 = new FXComboBox(vframe,8,&target_theme,FXDataTarget::ID_VALUE,COMBOBOX_INSERT_LAST|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  combobox1->setNumVisible(6);
  combobox1->appendItem("dark");
  combobox1->appendItem("shine");
  combobox1->appendItem("classic");
  combobox1->appendItem("gentle");
  combobox1->appendItem("fog");
  combobox1->appendItem("moon");

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);

  FXMatrix *themesmatrix=new FXMatrix(vframe,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5,5,5);
  new FXLabel(themesmatrix,"Dark",darkthemeicon,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X|ICON_ABOVE_TEXT);
  new FXLabel(themesmatrix,"Gentle",gentlethemeicon,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X|ICON_ABOVE_TEXT);
  new FXLabel(themesmatrix,"Shine",shinethemeicon,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X|ICON_ABOVE_TEXT);
  new FXLabel(themesmatrix,"Fog",fogthemeicon,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X|ICON_ABOVE_TEXT);
  new FXLabel(themesmatrix,"Classic",classicthemeicon,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X|ICON_ABOVE_TEXT);
  new FXLabel(themesmatrix,"Moon",moonthemeicon,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X|ICON_ABOVE_TEXT);


  new FXSeparator(hframe,SEPARATOR_GROOVE|LAYOUT_FILL_Y);

  vframe = new FXVerticalFrame(hframe,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(vframe," Language selection",NULL,LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);

  combobox2 = new FXComboBox(vframe,8,&target_language,FXDataTarget::ID_VALUE,COMBOBOX_INSERT_LAST|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_CENTER_X);
  combobox2->setNumVisible(6);
  combobox2->appendItem("en");
  combobox2->appendItem("it");
  combobox2->appendItem("de");
  combobox2->appendItem("fr");
  combobox2->appendItem("es");
  combobox2->appendItem("ru");

  new FXSeparator(vframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);



  new FXSeparator(mainframe,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  FXHorizontalFrame *copyright=new FXHorizontalFrame(mainframe,LAYOUT_BOTTOM|LAYOUT_FILL_X,0,0,0,0,5,5,5,5);
  label = new FXLabel(copyright,"FXMovieManager Settings - Copyright (C) Luigi Ziviani",NULL,LAYOUT_FILL_X|LAYOUT_CENTER_Y|JUSTIFY_LEFT,0,0,0,0,15);
  // set the same look of separators
  label->disable();

  new FXButton(copyright,"&Quit and Save?",NULL,this,FXTopWindow::ID_CLOSE,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0,20,20);

  }


// Delete window
MimeTypeDialog::~MimeTypeDialog(){
  delete titlefont;
  delete logoicon;
  delete abouticon;
  delete conficon;
  delete diricon;
  delete darkthemeicon;
  delete shinethemeicon;
  delete classicthemeicon;
  delete gentlethemeicon;
  delete fogthemeicon;
  delete moonthemeicon;
  }


// Search for Icon and load it
static FXIcon *createIconApp(FXApp *app,const FXString& name,const FXString& iconpath){
  FXIconSource iconsource(app);
  FXIcon *icon=NULL;
  FXString fullpath;
  if(FXStat::exists(name)){
    fullpath=name;
    }
  else{
    fullpath=FXPath::search(iconpath,name);
    if(fullpath.empty()) return NULL;
    }
  icon=iconsource.loadIconFile(fullpath);
  if(icon){
    icon->blend(app->getBaseColor());
    icon->create();
    return icon;
    }
  return NULL;
  }


// Update Icon
static void updateIconApp(FXApp* app,FXButton* button,const FXString& path){
  FXIconSource iconsource(app);
  FXIcon *icon=NULL;
  if(button->getIcon()){
    icon=button->getIcon();
    button->setIcon(NULL);
    delete icon;
    icon=NULL;
    }
  icon=iconsource.loadIconFile(path);
  if(icon){
    icon->blend(app->getBaseColor());
    icon->create();
    button->setIcon(icon);
    }
  }


// Save file association entry
void MimeTypeDialog::saveFileAssociation(){
  if(fileassociation.extension.empty()) return;
  FXString added;
  added=fileassociation.command+";"+fileassociation.type+";"+fileassociation.bigname+";"+fileassociation.smallname+";"+fileassociation.mimetype;
  getApp()->reg().writeStringEntry("FILETYPES",fileassociation.extension.text(),added.text());
  }


// Save theme colors
FXbool MimeTypeDialog::setThemeColors(){
  theme=combobox1->getText();

  if(theme=="dark"){
  	theme_fore     = FXRGB(150,150,150);
  	theme_base     = FXRGB( 60, 60, 60);
  	theme_back     = FXRGB( 20, 20, 20);
  	theme_menufore = FXRGB(150,150,150);
  	theme_menuback = FXRGB( 10, 10, 10);
  	theme_selfore  = FXRGB(200,200,200);
  	theme_selback  = FXRGB(100,100,100);
	return TRUE;
	}
  else if(theme=="shine"){
  	theme_fore     = FXRGB( 50, 50, 50);
  	theme_base     = FXRGB(230,230,230);
  	theme_back     = FXRGB(250,250,250);
  	theme_menufore = FXRGB( 50, 50, 50);
  	theme_menuback = FXRGB(180,180,180);
  	theme_selfore  = FXRGB( 50, 50, 50);
  	theme_selback  = FXRGB(200,200,200);
	return TRUE;
	}
  else if(theme=="classic"){
  	theme_fore     = FXRGB(  0,  0,  0);
  	theme_base     = FXRGB(180,180,180);
  	theme_back     = FXRGB(210,210,210);
  	theme_menufore = FXRGB(  0,  0,  0);
  	theme_menuback = FXRGB(130,130,130);
  	theme_selfore  = FXRGB(  0,  0,  0);
  	theme_selback  = FXRGB(150,150,150);
	return TRUE;
	}
  else if(theme=="gentle"){
  	theme_fore     = FXRGB(  0,  0,  0);
  	theme_base     = FXRGB(160,160,160);
  	theme_back     = FXRGB(230,230,230);
  	theme_menufore = FXRGB(  0,  0,  0);
  	theme_menuback = FXRGB(200,200,200);
  	theme_selfore  = FXRGB(  0,  0,  0);
  	theme_selback  = FXRGB(180,180,180);
	return TRUE;
 	}
  else if(theme=="fog"){
  	theme_fore     = FXRGB(  0,  0,  0);
  	theme_base     = FXRGB(160,160,160);
  	theme_back     = FXRGB(230,230,230);
  	theme_menufore = FXRGB(  0,  0,  0);
  	theme_menuback = FXRGB(200,200,200);
  	theme_selfore  = FXRGB(  0,  0,  0);
  	theme_selback  = FXRGB(180,180,180);
	return TRUE;
	}
  else if(theme=="moon"){
  	theme_fore     = FXRGB(150,150,150);
  	theme_base     = FXRGB( 60, 60, 60);
  	theme_back     = FXRGB( 20, 20, 20);
  	theme_menufore = FXRGB(150,150,150);
  	theme_menuback = FXRGB( 10, 10, 10);
  	theme_selfore  = FXRGB(200,200,200);
  	theme_selback  = FXRGB(100,100,100);
	return TRUE;
	}
  else
	return FALSE;
}


// Save file association entry
void MimeTypeDialog::saveThemeColors(){
  // Save Colors
  getApp()->reg().writeColorEntry("SETTINGS","forecolor",theme_fore);
  getApp()->reg().writeColorEntry("SETTINGS","basecolor",theme_base);
  getApp()->reg().writeColorEntry("SETTINGS","backcolor",theme_back);
  getApp()->reg().writeColorEntry("SETTINGS","selmenutextcolor",theme_menufore);
  getApp()->reg().writeColorEntry("SETTINGS","selmenubackcolor",theme_menuback);
  getApp()->reg().writeColorEntry("SETTINGS","selforecolor",theme_selfore);
  getApp()->reg().writeColorEntry("SETTINGS","selbackcolor",theme_selback);
}


// Set search path for icons
long MimeTypeDialog::onCmdSelectIconPath(FXObject*,FXSelector,void*){
  FXString path=(iconpath.empty()) ? "/usr/share/" : iconpath.text();
  FXDirDialog opendialog(this,"Search path for icons?");
  opendialog.setDirectory(path);
  if(opendialog.execute()){
    iconpath=opendialog.getDirectory();
    }
  return 1;
  }


// Set command to open the file with
long MimeTypeDialog::onCmdSelectCommand(FXObject*,FXSelector,void*){
  FXString path=(fileassociation.command.empty()) ? "/usr/bin/" : fileassociation.command.text();
  FXString search=FXFileDialog::getOpenFilename(this,"Command to open file with?",path,"*");
  if(!search.empty()){
    fileassociation.command=search;
    }
  return 1;
  }


long MimeTypeDialog::onCmdSelectMimeType(FXObject*,FXSelector,void*ptr){
  FXString content=(FXchar*)ptr;
  if(!content.empty() && (mimetypelist->findItem(content)==-1)){
    mimetypelist->appendItem(content);
    mimetypelist->sortItems();
    }
  fileassociation.mimetype=content;
  return 1;
  }


long MimeTypeDialog::onCmdFileAssociation(FXObject*,FXSelector,void*){
  saveFileAssociation();

  FXIcon *newicon;
  FXString iconname;
  FXString association;

  fileassociation.extension=associationlist->getItemText(associationlist->getCurrentItem());
  if(fileassociation.extension.empty()) return 1;

  association=getApp()->reg().readStringEntry("FILETYPES",fileassociation.extension.text());

  // here is the same association order of ControlPanel to be able to import associations 
  fileassociation.command = association.section(";",0);  // the command to open the file with
  fileassociation.type = association.section(";",1);     // the name of the type
  fileassociation.bigname = association.section(";",2);  // big icon 
  fileassociation.smallname = association.section(";",3); // small icon 
  fileassociation.mimetype = association.section(";",4); // content-type [object/format]

  if(!fileassociation.mimetype.empty()){
    FXint no=mimetypelist->findItem(fileassociation.mimetype);
    mimetypelist->setCurrentItem(no);
    }
  else {
    FXint no=mimetypelist->findItem(" ");
    mimetypelist->setCurrentItem(no);
    }

  if(button_bigname->getIcon()){
    newicon=button_bigname->getIcon();
    button_bigname->setIcon(NULL);
    delete newicon;
    }
  newicon=createIconApp(getApp(),fileassociation.bigname,iconpath);
  button_bigname->setIcon(newicon);

  if(button_smallname->getIcon()){
    newicon=button_smallname->getIcon();
    button_smallname->setIcon(NULL);
    delete newicon;
    }
  newicon=createIconApp(getApp(),fileassociation.smallname,iconpath);
  button_smallname->setIcon(newicon);

  return 1;
  }


long MimeTypeDialog::onCmdCreateFileAssociation(FXObject*,FXSelector,void*){
  FXString result;
  if(FXInputDialog::getString(result,this,"New File Association","Please enter fileassociation extension:",NULL)){
    if(result.empty()) return 1;
    if((associationlist->findItem(result))>=0){
      FXMessageBox::question(this,MBOX_OK,"Duplicate Association","The extension given %s already exists.",result.text());
      return 1;
      }

    // Save Old
    saveFileAssociation();

    fileassociation.extension = result;
    fileassociation.type = "";
    fileassociation.command = "";
    fileassociation.mimetype = "";
    fileassociation.bigname = "";
    fileassociation.smallname = "";

    // Save New
    saveFileAssociation();

    FXint no = associationlist->appendItem(fileassociation.extension);
    associationlist->setCurrentItem(no,TRUE);
    associationlist->sortItems();
    associationlist->makeItemVisible(associationlist->getCurrentItem());
    }
  return 1;
  }


long MimeTypeDialog::onCmdDeleteFileAssociation(FXObject*,FXSelector,void*){
  if(fileassociation.extension.empty()) return 1;
  if(FXMessageBox::question(this,MBOX_OK_CANCEL,"Delete Fileassociation?","Are you sure you want to delete\nthe fileassociation for %s",fileassociation.extension.text())==MBOX_CLICKED_OK){
    getApp()->reg().deleteEntry("FILETYPES",fileassociation.extension.text());
    fileassociation.extension="";
    associationlist->removeItem(associationlist->getCurrentItem(),TRUE);
    }
  return 1;
  }


long MimeTypeDialog::onCmdRenameFileAssociation(FXObject*,FXSelector,void*){
  FXint no;
  FXString result=fileassociation.extension;
  if(FXInputDialog::getString(result,this,"Rename File Association","Rename File Association",NULL)){
    if(result==fileassociation.extension) return 1;
    if((no=associationlist->findItem(result))>=0){
      if(FXMessageBox::question(this,MBOX_YES_NO,"Replace Association?","Would you like to replace the existing association?")!=MBOX_CLICKED_YES){
        return 1;
        }
      }

    // Find Extension to rename
    getApp()->reg().deleteEntry("FILETYPES",fileassociation.extension.text());
    fileassociation.extension=result;
    // Save renamed Extension 
    saveFileAssociation();
    if(no>=0){
      FXint olditem = associationlist->getCurrentItem();
      associationlist->setCurrentItem(no,TRUE);
      // Remove old Extension
      associationlist->removeItem(olditem);
      }
    else {
      associationlist->setItemText(associationlist->getCurrentItem(),fileassociation.extension);
      }
    }
  return 1;
  }


long MimeTypeDialog::onCmdSelectIcon(FXObject*,FXSelector sel,void*){
  FXString name;
    switch(FXSELID(sel)){
	case ID_SELECT_ICONBIG_NAME: name=fileassociation.bigname; break;
	case ID_SELECT_ICONSMALL_NAME: name=fileassociation.smallname; break;
    }

    if(!FXStat::exists(name)){ name=FXPath::search(iconpath,name); }

    if(name.empty()){
      if(!iconpath.empty()){ name=iconpath.section(PATHLISTSEP,0)+PATHSEPSTRING; }
      else{ name=FXSystem::getCurrentDirectory(); }
    }

  FXFileDialog opendialog(this,"Select the Icon");
  opendialog.setSelectMode(SELECTFILE_EXISTING);
  opendialog.setFilename(name);
  opendialog.showImages(TRUE);
  if(opendialog.execute()){

	FXString iconroot=opendialog.getFilename();
	FXString iconname=FXPath::name(iconroot);

	switch(FXSELID(sel)){
		case ID_SELECT_ICONBIG_NAME:
			updateIconApp(getApp(),button_bigname,iconroot);
			fileassociation.bigname=iconname;
		break;
		case ID_SELECT_ICONSMALL_NAME:
			updateIconApp(getApp(),button_smallname,iconroot);
			fileassociation.smallname=iconname;
		break;
	}
    }
  return 1;
  }



// Close the dialog
long MimeTypeDialog::onCmdCloseDialog(FXObject*,FXSelector,void*){

  FXint result=FXMessageBox::question(this,MBOX_SAVE_CANCEL_DONTSAVE,"Save Changes?","Do you want to save changes to the FXMovieManager Registry?\n\nThen you must restart the application for the changes to take effect.");
  if(result==MBOX_CLICKED_CANCEL){ 
	return 1;
  }
  if(result==MBOX_CLICKED_SAVE){

	saveFileAssociation();
	setThemeColors();
	saveThemeColors();
	// Save iconpath,theme and language Settings
	getApp()->reg().writeStringEntry("SETTINGS","iconpath",iconpath.text());
  	getApp()->reg().writeStringEntry("SETTINGS","theme",theme.text());
	language=combobox2->getText();
  	getApp()->reg().writeStringEntry("SETTINGS","language",language.text());
  }
  FXWindow::hide();
  return 1;
  }

// EOF
